{//////////////////////////////////////////////////////////////////////////}
{//                                                                      //}
{//   GPCC: - a compiler front end for GNU Pascal (GPC)                  //}
{//   v1.00 - by Dr Abimbola A. Olowofoyeku (The African Chief)          //}
{//       - released as part of the GNU Pascal project -                 //}
{//       - automatically searches for GPCC.CFG - to read                //}
{//       - parameter lists therefrom                                    //}
{//                                                                      //}
{//       COMPILES BOTH UNDER BORLAND PASCAL AND GPCC                    //}
{//                                                                      //}
{//  This program is a free program; you can redistribute and/or modify  //}
{//  it under the terms of the GNU Library General Public License,       //}
{//  version 2, as published by the Free Software Foundation.            //}
{//                                                                      //}
{//  This library is distributed in the hope that it will be useful,     //}
{//  but WITHOUT ANY WARRANTY; without even the implied warranty of      //}
{//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       //}
{//  GNU Library General Public License for more details.                //}
{//                                                                      //}
{//  You should have received a copy of the GNU Library General Public   //}
{//  License along with this library (see the file COPYING.LIB);         //}
{//  if not, write to the Free Software Foundation, Inc., 675 Mass Ave,  //}
{//  Cambridge, MA 02139, USA.                                           //}
{//                                                                      //}
{//  May 1997 - Dr A Olowofoyeku: laa12@keele.ac.uk                      //}
{//  Homepage: http://ourworld.compuserve.com/homepages/African_Chief    //}
{//////////////////////////////////////////////////////////////////////////}
PROGRAM GPCC;

{$ifdef ver70}    { Borland v7.x }
{$F+,I-,S-,G+}
   USES
   TPDOS, TPSTRING;    { BP - requires TurboPower units }
   {$M 4096,0,65536}
{$else ver70}     { GPC }
   USES GPCTYPES, SYSTEM, STRINGS, GPCUTIL;
   {$X+}    { use extended syntax }
   {$W-}    { disable warnings }
{$endif ver70}

{ array to hold parameters }
Type
ParmStr  = array [0..127] of String [80]; { let's go overboard! }
pParmStr = ^ParmStr;

Var
s, s1  : String;
s2, s3 : String;
Tmp    : String;
p      : pParmStr;
i, j, k : integer;
t      : text;
isunit : boolean;

Begin

   Tmp := '';
   Write ( 'GPCC (' ); Write ( {$ifdef ver70}'BP' {$else} 'GPC' {$endif} );
   Writeln ( ' version), 1.00: compiler front end for GPC (GNU Pascal).' );
   Writeln ( 'Released as part of the GNU Pascal project.' );
   Writeln ( 'May 1997, Dr. A. Olowofoyeku (The African Chief).' );
   Writeln;

   If ParamCount < 1 then begin
      Writeln ( 'Syntax = GPCC <SOURCEFILE[.PAS]> [parameters] [!!logfile.log]' );
      Writeln ( '         The parameters can be put in GPCC.CFG' );
      Writeln ( '         "!!logfile.log" redirects output to "logfile.log"' );
      Halt;
   end;

   New ( p );
   FillChar ( p^, Sizeof ( p^ ), Chr ( 0 ) );
   p^ [0] := ParamStr ( 0 ); { store the program executable }

   { process the parameters }
   for j := 2 to ParamCount do begin
       s2 := ParamStr ( j );
       { output redirection; "!!filename.log" == call REDIR }
       if ( s2 [1] = '!' ) and ( s2 [2] = '!' )
       then begin
          Delete ( s2, 1, 2 );
          Tmp := s2;
       end else p^ [j] := s2;
   end;

   s2 := paramstr ( 1 );
   s  := justextension ( s2 );
   if ( s = '' ) then begin
      s2 := forceextension ( s2, 'pas' );
   end;

   { check whether file exists }
   If not ExistFile ( s2 ) then begin
      Writeln ( 'I cannot find ', s2 );
      Halt;
   end;

   { check whether it is a unit }
   isunit := false;
   s3 := justfilename ( s2 );
   s3 := stupcase ( forceextension ( s2, '' ) );
   i  := length ( s3 );
   if s3 [i] = '.' then delete ( s3, i, 1 );
   Assign ( t, s2 );
   Reset ( t );
   i := 0;
   if ioresult = 0 then begin
      while not eof ( t ) do begin
          Readln ( t, s1 );
          s1 := trim ( stupcase ( s1 ) );

          if ( ( pos ( 'UNIT', s1 ) = 1 )
          or ( ( pos ( 'MODULE', s1 ) = 1 ) )
          and  ( pos ( s3, s1 ) > 4 ) )
          then begin
             isunit := true;
             break;
          end;

          if
          ( 
              ( ( pos ( 'PROGRAM', s1 ) = 1 )
          or  (   pos ( 'LIBRARY', s1 ) = 1 ) )
          and (   pos ( s3, s1 ) > 7 )
          )
          then begin
             isunit := false;
             break;
          end;
          inc ( i );
          if i > 64 then break; {process up to 64 lines}
      end;{not eof}
      close ( t ); if ioresult <> 0 then;
   end;{ioresult}


   s3 := s2;

   if isunit { is it a unit ? }
   then s := ' -c '
   else s := ' -o ' + forceextension ( s2, 'exe' ) + ' ';

   s2 := s + s2;

   { look for cfg file }
   s  := 'GPCC.CFG';
   s1 := AddBackSlash ( JustPathName ( p^ [0] ) ) + 'GPCC.CFG';

   if   ExistFile ( s1 ) then s := s1
   else if ExistFile ( s ) then s := FullPathName ( s )
   else if ExistOnPath ( s, s ) then {}
   else s := '';

   If ( s <> '' )
   then begin
      Assign ( t, s );
      Reset ( t );
      If Ioresult = 0 then begin
         while not eof ( t ) do begin
             Readln ( t, s1 );
             s1  := trim ( s1 );
             { ';' and '#' are comment lines }
             if ( s1 <> '' ) and ( s1 [1] <> ';' ) and ( s1 [1] <> '#' )
	     then begin
                inc ( j );
                p^ [j] := s1;
             end;{s1 > ''}
         end; {not eof(t) }
         Close ( t ); if ( ioresult <> 0 ) then;
      end; {ioresult=0}
   end; {s > ''}

   { set the executable name }
   If Tmp <> '' then s := 'redir -o ' + Tmp + ' -eo gpc -v' { call "redir" }
   else
   s  := 'gpc';

   s1 := '';
   if paramcount > 1 then k := 2 else k := 1;
   for i := k to j do begin
      s1 := s1 + p^ [i] + ' ';
   end;
   
   Dispose ( p );

   s := s + ' ' + trim ( s1 ) + s2;

   If isunit
   then Writeln ( 'Compiling UNIT/MODULE "', s3, '" ... ' )
   else Writeln ( 'Compiling PROGRAM/DLL "', s3, '" ... ' );

   i := ExecDos ( s, True, Nil );
   {Writeln ( s ); }
End.

